<?php

namespace Factory;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Teknisa\Libs\Util\Libraries\PHPMailer;
use Teknisa\Libs\Util\Libraries\SMTP;
use \Util\mail_template;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\Libs\Util\Address;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\ORM\Platform\OraclePlatform;
use Doctrine\DBAL\DriverManager;
use Exception; 

class CnpjFactory {
    
    protected $connection;
    protected $address;
    
    public function __construct(Environment $environment, NewCode $newCode, EntityManager $entityManager)  {
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        $this->newCode = $newCode;
        $this->entityManager = $entityManager;
    }

	/* GETs e SETs */
	private function getCNPJBase($cnpj){
        try {
            $sql = "SELECT DOCUMENTO,
                                  RAZAOSOCIAL,
                                  NOMEFANTASIA,
                                  CODIGOATIVIDADEECONOMICA,
                                  CODIGOATIVIDADEECONOMICADESC,
                                  INSCRICAOESTADUAL
                                FROM PESSOAJURIDICANFE
                                WHERE DOCUMENTO = :P_DOCUMENTO";
            $params = array(
                'P_DOCUMENTO' => $cnpj
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function getEnderecoCNPJ($cnpj){
        try {
            $sql = "SELECT DOCUMENTO,
                           CEP,
                           LOGRADOURO,
                           NUMERO,
                           COMPLEMENTO,
                           BAIRRO,
                           CIDADE,
                           ESTADO,
                           TIPO
                    FROM PESSOAJURIDICANFE_ENDERECO
                    WHERE DOCUMENTO = :P_DOCUMENTO";
            $params = array(
                'P_DOCUMENTO' => $cnpj
            );
            return $this->entityManager->getConnection()->fetchAll($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function setCNPJBase($data){
        $dadosCnpj = $data['PessoaJuridicaEstendidaResult'];
        
        try {
            $insert = "INSERT INTO PESSOAJURIDICANFE (DOCUMENTO, 
                                                     RAZAOSOCIAL, 
                                                     NOMEFANTASIA, 
                                                     CODIGOATIVIDADEECONOMICA, 
                                                     CODIGOATIVIDADEECONOMICADESC,
                                                     INSCRICAOESTADUAL)
                        VALUES(:P_DOCUMENTO, 
                               :P_RAZAOSOCIAL, 
                               :P_NOMEFANTASIA, 
                               :P_CODIGOATIVIDADEECONOMICA, 
                               :P_CODIGOATIVIDADEECONOMICADESC,
                               :P_INSCRICAOESTADUAL)";
            
            $params = array(':P_DOCUMENTO'                   => $dadosCnpj['Documento'],
                            ':P_RAZAOSOCIAL'                 => $dadosCnpj['RazaoSocial'],
                            ':P_NOMEFANTASIA'                => $dadosCnpj['NomeFantasia'],
                            ':P_CODIGOATIVIDADEECONOMICA'    => $dadosCnpj['CodigoAtividadeEconomica'],
                            ':P_CODIGOATIVIDADEECONOMICADESC'=> $dadosCnpj['CodigoAtividadeEconomicaDescricao'],
                            ':P_INSCRICAOESTADUAL'           => $dadosCnpj['InscricaoEstadual']);

            return $this->entityManager->getConnection()->executeUpdate($insert, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function setEnderecoCNPJ($data){
        $dadosCnpj = $data['PessoaJuridicaEstendidaResult'];
        
        try {
            $insert = "INSERT INTO PESSOAJURIDICANFE_ENDERECO (DOCUMENTO,
                                                               CEP,
                                                               LOGRADOURO,
                                                               NUMERO,
                                                               COMPLEMENTO,
                                                               BAIRRO,
                                                               CIDADE,
                                                               ESTADO,
                                                               TIPO)
                        VALUES (:P_DOCUMENTO,
                                :P_CEP,
                                :P_LOGRADOURO,
                                :P_NUMERO,
                                :P_COMPLEMENTO,
                                :P_BAIRRO,
                                :P_CIDADE,
                                :P_ESTADO,
                                :P_TIPO)";
            
            $params = array(':P_DOCUMENTO'  => $dadosCnpj['Documento'],
                            ':P_CEP'        => $dadosCnpj['Enderecos']['Endereco']['CEP'],
                            ':P_LOGRADOURO' => $dadosCnpj['Enderecos']['Endereco']['Logradouro'],
                            ':P_NUMERO'     => $dadosCnpj['Enderecos']['Endereco']['Numero'],
                            ':P_COMPLEMENTO'=> $dadosCnpj['Enderecos']['Endereco']['Complemento'],
                            ':P_BAIRRO'     => $dadosCnpj['Enderecos']['Endereco']['Bairro'],
                            ':P_CIDADE'     => $dadosCnpj['Enderecos']['Endereco']['Cidade'],
                            ':P_ESTADO'     => $dadosCnpj['Enderecos']['Endereco']['Estado'],
                            ':P_TIPO'       => $dadosCnpj['Enderecos']['Endereco']['Tipo']);

            return $this->entityManager->getConnection()->executeUpdate($insert, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    /* GETs e SETs */
    
    public function consultaCnpj($nrinsjurclie){
        try {
            $dadosCnpj = $this->consultaCNPJBase($nrinsjurclie);
      	    // if dadoscnpj OU empresa não tenha endereço, chama atualiza cnpj que apaga qualquer vestigio antigo e lança novo
      	    if(!$dadosCnpj){
      	    	$dadosCnpjGov = $this->consultaCNPJGov($nrinsjurclie);
      	    	$this->setCNPJBase($dadosCnpjGov);
      	    	$this->setEnderecoCNPJ($dadosCnpjGov);
      	    	$dadosCnpj = $dadosCnpjGov;
      	    }
      		return $dadosCnpj;
        } catch (\Exception $e) {
            throw $e;
        }
  	}
  	
    private function consultaCNPJBase($nrinsjurclie){
        # VERIFICA SE CNPJ CONSULTADO JÁ EXISTE NA BASE DE DADOS
        $dadosCnpj = $this->getCNPJBase($nrinsjurclie);
        if($dadosCnpj){
        	$dadosCnpj = $dadosCnpj[0];
        	# BUSCA ENDEREÇO CADASTRADO PARA O CNPJ
        	$endereco = $this->getEnderecoCNPJ($nrinsjurclie);
        	if (isset($endereco)) $endereco = $endereco[0];
            $dataReturn = array(
                            "PessoaJuridicaEstendidaResult" => array(
                                "Documento"                         => $dadosCnpj['DOCUMENTO'],
                                "RazaoSocial"                       => $dadosCnpj['RAZAOSOCIAL'],
                                "NomeFantasia"                      => $dadosCnpj['NOMEFANTASIA'],
                                "CodigoAtividadeEconomica"          => $dadosCnpj['CODIGOATIVIDADEECONOMICA'],
                                "CodigoAtividadeEconomicaDescricao" => $dadosCnpj['CODIGOATIVIDADEECONOMICADESC'],
                                "InscricaoEstadual"                 => $dadosCnpj['INSCRICAOESTADUAL'],
                                "Enderecos" => array(
                                    "Endereco" => array(
                                                    "Tipo"          => isset($endereco['TIPO'])         ? $endereco['TIPO'] : '',
                                                    "Logradouro"    => isset($endereco['LOGRADOURO'])   ? $endereco['LOGRADOURO'] : '',
                                                    "Numero"        => isset($endereco['NUMERO'])       ? $endereco['NUMERO'] : '',
                                                    "Complemento"   => isset($endereco['COMPLEMENTO'])  ? $endereco['COMPLEMENTO'] : '',
                                                    "Bairro"        => isset($endereco['BAIRRO'])       ? $endereco['BAIRRO'] : '',
                                                    "Cidade"        => isset($endereco['CIDADE'])       ? $endereco['CIDADE'] : '',
                                                    "Estado"        => isset($endereco['ESTADO'])       ? $endereco['ESTADO'] : '',
                                                    "CEP"           => isset($endereco['CEP'])          ? $endereco['CEP'] : ''
                                                )
                                ),
                                "Status" => true,
                                "Transacao" => array(
                                    "Status" => true,
                					"CodigoStatus" => "G000M001",
                					"CodigoStatusDescricao" => "Transacao realizada com sucesso"
                                )
                            )
                        );
            
            return $dataReturn;
        }else{
            return $dadosCnpj;
        }
    }

	private function consultaCNPJGov($NrInscricao){ 
	    #url produção
		$client = new \SoapClient('http://www.soawebservices.com.br/webservices/producao/cdc/cdc.asmx?WSDL');
		#url teste
        // $client = new \SoapClient('http://www.soawebservices.com.br/webservices/test-drive/cdc/cdc.asmx?WSDL');
		$function = "PessoaJuridicaEstendida";
		$arguments = array(array('Credenciais' => array('Email' 	=> 'administrativo@teknisa.com',
                                             	 		'Senha' 	=> 'K1*#n2'),
    	                    							'Documento' => $NrInscricao));
        #url produção
		$options = array('location' => 'http://www.soawebservices.com.br/webservices/producao/cdc/cdc.asmx');
		#url teste
        // $options = array('location' => 'http://www.soawebservices.com.br/webservices/test-drive/cdc/cdc.asmx');

        try {
            $data = $client->__soapCall($function, $arguments, $options);
        }catch (\Exception $e)
        {
            return  array("Status" => false, "CONEXAO" => true);
        }
        
        $data = $this->obj_to_array($data);

        return $data;
  	}
  	
  	private function obj_to_array($data){
  	    if (is_array($data) || is_object($data)){
  	        $result = array();
  	        foreach ($data as $key => $value){
  	            $result[$key] = $this->obj_to_array($value);
  	        }
  	        return $result;
  	    }
  	    return $data;
    }

}